@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Andreas Fabri
@@ -23,10 +24,12 @@
 
 #include <CGAL/license/Polygon_mesh_processing/measure.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/boost/graph/iterator.h>
 #include <CGAL/boost/graph/helpers.h>
 #include <CGAL/boost/graph/properties.h>
+#include <boost/unordered_set.hpp>
 #include <boost/graph/graph_traits.hpp>
 
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
@@ -34,6 +37,10 @@
 #include <CGAL/squared_distance_3.h>
 #include <CGAL/Kernel/global_functions_3.h>
 
+#include <CGAL/Lazy.h> // needed for CGAL::exact(FT)/CGAL::exact(Lazy_exact_nt<T>)
+
+#include <utility>
+
 #ifdef DOXYGEN_RUNNING
 #define CGAL_PMP_NP_TEMPLATE_PARAMETERS NamedParameters
 #define CGAL_PMP_NP_CLASS NamedParameters
@@ -41,6 +48,8 @@
 
 namespace CGAL {
 
+namespace Polygon_mesh_processing {
+
 // workaround for area(face_range,tm) overload
 template<typename CGAL_PMP_NP_TEMPLATE_PARAMETERS, typename NP>
 class GetGeomTraits<CGAL_PMP_NP_CLASS, NP>
@@ -49,19 +58,17 @@ public:
   struct type{};
 };
 
-namespace Polygon_mesh_processing {
-
   /**
   * \ingroup measure_grp
   * computes the length of an edge of a given polygon mesh.
   * The edge is given by one of its halfedges, or the edge itself.
   *
   * @tparam PolygonMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param h one halfedge of the edge to compute the length
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
   *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
@@ -71,7 +78,7 @@ namespace Polygon_mesh_processing {
   * \cgalNamedParamsEnd
   *
   * @return the length of `h`. The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -96,7 +103,7 @@ namespace Polygon_mesh_processing {
     using boost::get_param;
 
     typename GetVertexPointMap<PolygonMesh, NamedParameters>::const_type
-    vpm = choose_param(get_param(np, vertex_point),
+    vpm = choose_param(get_param(np, internal_np::vertex_point),
                        get_const_property_map(CGAL::vertex_point, pmesh));
 
     return CGAL::approximate_sqrt(CGAL::squared_distance(get(vpm, source(h, pmesh)),
@@ -138,11 +145,11 @@ namespace Polygon_mesh_processing {
   * that contains a given halfedge.
   *
   * @tparam PolygonMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param h a halfedge of the border polyline of which the length is computed
   * @param pmesh the polygon mesh to which `h` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
   *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
@@ -153,7 +160,7 @@ namespace Polygon_mesh_processing {
   *
   * @return the length of the sequence of border edges of `face(h, pmesh)`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `pmesh`.
   *
@@ -196,6 +203,80 @@ namespace Polygon_mesh_processing {
       CGAL::Polygon_mesh_processing::parameters::all_default());
   }
 
+  /**
+  * \ingroup measure_grp
+  * finds the longest border of a given triangulated surface and returns
+  * a halfedge that is part of this border and the length of this border.
+  *
+  * @tparam PolygonMesh a model of `HalfedgeGraph`
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+  *
+  * @param pmesh the polygon mesh
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+  *
+  * \cgalNamedParamsBegin
+  *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+  *   If this parameter is omitted, an internal property map for
+  *   `CGAL::vertex_point_t` should be available in `PolygonMesh`\cgalParamEnd
+  *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+  * \cgalNamedParamsEnd
+  *
+  * @return a pair composed of two members:
+  *   - `first`: a halfedge on the longest border.
+  *     The return type `halfedge_descriptor` is a halfedge descriptor. It is
+  *     deduced from the graph traits corresponding to the type `PolygonMesh`.
+  *   - `second`: the length of the longest border
+  *     The return type `FT` is a number type. It is
+  *     either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
+  *     or the geometric traits class deduced from the point property map
+  *     of `pmesh`
+  *
+  */
+  template<typename PolygonMesh,
+           typename NamedParameters>
+#ifdef DOXYGEN_RUNNING
+  std::pair<halfedge_descriptor, FT>
+#else
+  std::pair<typename boost::graph_traits<PolygonMesh>::halfedge_descriptor,
+            typename GetGeomTraits<PolygonMesh, NamedParameters>::type::FT>
+#endif
+  longest_border(const PolygonMesh& pmesh
+                 , const NamedParameters& np)
+  {
+    typedef typename CGAL::Kernel_traits<typename property_map_value<PolygonMesh,
+                                                                     CGAL::vertex_point_t>::type>::Kernel::FT  FT;
+    typedef typename boost::graph_traits<PolygonMesh>::halfedge_descriptor halfedge_descriptor;
+
+    boost::unordered_set<halfedge_descriptor> visited;
+    halfedge_descriptor result_halfedge = boost::graph_traits<PolygonMesh>::null_halfedge();
+    FT result_len = 0;
+    BOOST_FOREACH(halfedge_descriptor h, halfedges(pmesh)){
+      if(visited.find(h)== visited.end()){
+        if(is_border(h,pmesh)){
+          FT len = 0;
+          BOOST_FOREACH(halfedge_descriptor haf, halfedges_around_face(h, pmesh)){
+            len += edge_length(haf, pmesh, np);
+            visited.insert(haf);
+          }
+          if(result_len < len){
+            result_len = len;
+            result_halfedge = h;
+          }
+        }
+      }
+    }
+    return std::make_pair(result_halfedge, result_len); 
+  }
+  
+ template<typename PolygonMesh>  
+ std::pair<typename boost::graph_traits<PolygonMesh>::halfedge_descriptor,
+           typename CGAL::Kernel_traits<typename property_map_value<PolygonMesh,
+                                                                    CGAL::vertex_point_t>::type>::Kernel::FT>
+ longest_border(const PolygonMesh& pmesh)
+ {
+   return longest_border(pmesh,
+                         CGAL::Polygon_mesh_processing::parameters::all_default());
+ }
 
   /**
   * \ingroup measure_grp
@@ -203,11 +284,11 @@ namespace Polygon_mesh_processing {
   * triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param f the face of which the area is computed
   * @param tmesh the triangulated surface mesh to which `f` belongs
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
   *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
@@ -216,11 +297,11 @@ namespace Polygon_mesh_processing {
   *  \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
   * \cgalNamedParamsEnd
   *
-  *@pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
+  * @pre `f != boost::graph_traits<TriangleMesh>::%null_face()`
   *
   * @return the area of `f`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -243,7 +324,7 @@ namespace Polygon_mesh_processing {
     CGAL_precondition(boost::graph_traits<TriangleMesh>::null_face() != f);
 
     typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
-    vpm = choose_param(get_param(np, vertex_point),
+    vpm = choose_param(get_param(np, internal_np::vertex_point),
                        get_const_property_map(CGAL::vertex_point, tmesh));
 
     typedef typename boost::graph_traits<TriangleMesh>::halfedge_descriptor halfedge_descriptor;
@@ -277,11 +358,11 @@ namespace Polygon_mesh_processing {
           model of `Range`.
           Its iterator type is `InputIterator`.
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param face_range the range of faces of which the area is computed
   * @param tmesh the triangulated surface mesh to which the faces of `face_range` belong
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
   *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
@@ -292,7 +373,7 @@ namespace Polygon_mesh_processing {
   *
   * @return sum of face areas of `faces`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -337,10 +418,10 @@ namespace Polygon_mesh_processing {
   * computes the surface area of a triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param tmesh the triangulated surface mesh
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * \cgalNamedParamsBegin
   *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
@@ -351,7 +432,7 @@ namespace Polygon_mesh_processing {
   *
   * @return the surface area of `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   *
@@ -388,10 +469,10 @@ namespace Polygon_mesh_processing {
   * a closed triangulated surface mesh.
   *
   * @tparam TriangleMesh a model of `HalfedgeGraph`
-  * @tparam NamedParameters a sequence of \ref namedparameters
+  * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
   *
   * @param tmesh the closed triangulated surface mesh bounding the volume
-  * @param np optional sequence of \ref namedparameters among the ones listed below
+  * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
   *
   * @pre `tmesh` is closed
   *
@@ -404,7 +485,7 @@ namespace Polygon_mesh_processing {
   *
   * @return the volume bounded by `tmesh`.
   * The return type `FT` is a number type. It is
-  * either deduced from the `geom_traits` \ref namedparameters if provided,
+  * either deduced from the `geom_traits` \ref pmp_namedparameters "Named Parameters" if provided,
   * or the geometric traits class deduced from the point property map
   * of `tmesh`.
   */
@@ -424,7 +505,7 @@ volume(const TriangleMesh& tmesh, const CGAL_PMP_NP_CLASS& np)
   using boost::get_param;
 
   typename GetVertexPointMap<TriangleMesh, CGAL_PMP_NP_CLASS>::const_type
-    vpm = choose_param(get_param(np, vertex_point),
+    vpm = choose_param(get_param(np, internal_np::vertex_point),
                        get_const_property_map(CGAL::vertex_point, tmesh));
   typename GetGeomTraits<TriangleMesh, CGAL_PMP_NP_CLASS>::type::Point_3
     origin(0, 0, 0);
@@ -459,4 +540,6 @@ volume(const TriangleMesh& tmesh, const CGAL_PMP_NP_CLASS& np)
 }
 }
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYGON_MESH_PROCESSING_MEASURE_H
